#include "MtchIter.h"
#include <string.h>

#include "stdio/mydirent.h"
#include "fnmatch/myfnmtch.h"
#include "pathsplt.h"

#define MIN(a,b)        ( ((a)<(b)) ? (a) : (b) )
#define MAX(a,b)        ( ((a)>(b)) ? (a) : (b) )

static int IsItDir (const char* path)
{
  DIR *d = opendir ((char*)path);
  if (d == NULL)
    return 0;
  closedir (d);
  return 1;
}

int MatchIterator (const char* pPath, size_t MaxNameLen,
                   tpMatchFxn pMatchFxn)
{
  char* pDir=NULL;
  char* pPattern=NULL;
  char* pFName=NULL;
  DIR *dir = NULL;
  struct dirent *ent;
  int res = 0;

  pDir = (char*)malloc(1+MaxNameLen);
  if (pDir == NULL)
  {
    res = -1;
    goto lend;
  }

  pPattern = (char*)malloc(1+MaxNameLen);
  if (pPattern == NULL)
  {
    res = -1;
    goto lend;
  }

  pFName = (char*)malloc(1+MaxNameLen);
  if (pFName == NULL)
  {
    res = -1;
    goto lend;
  }

  if
  (
    (strlen(pPath)+sizeof(ent->d_name) > MaxNameLen)
  )
  {
    res = -1;
    goto lend;
  }

  pathsplit (pPath, pDir, pPattern);

  if (IsItDir (pPath))
  {
    res = 1;
    goto lend;
  }

  if ((dir = opendir((char*)pDir)) == NULL)
  {
    res = -1;
    goto lend;
  }

  while ((ent = readdir(dir)) != NULL)
  {
    char aName[sizeof(ent->d_name)];
    int match;

    strcpy (aName, ent->d_name);
    if (strchr (aName, '.') == NULL)
      strcat (aName, ".");

    match =
      MIN
      (
        fnmatch (pPattern, ent->d_name, FNM_NOESCAPE|FNM_NOCASE),
        fnmatch (pPattern, aName, FNM_NOESCAPE|FNM_NOCASE)
      ) == 0;

    if (match)
    {
      char* p;
      struct stat StatBuf;

      strcpy (pFName, pDir);
      p = strrchr (pFName,':');
      if ((p==NULL) || ((p!=NULL) && p[1]))
      {
        p = MAX (strrchr (pFName,'\\'), strrchr (pFName,'/'));
        if ((p==NULL) || ((p!=NULL) && p[1]))
          strcat (pFName, "\\");
      }
      strcat (pFName, ent->d_name);

      stat (pFName, &StatBuf);

      pMatchFxn (pFName, ent->d_name, &StatBuf);
    }
  }

lend:
  if (dir != NULL)
    closedir (dir);

  if (pDir != NULL)
    free (pDir);
  if (pPattern != NULL)
    free (pPattern);
  if (pFName != NULL)
    free (pFName);

  return res;
}
