#ifndef _ALLOC_H_
#define _ALLOC_H_

#include <stddef.h>

#pragma pack (push, 1)

typedef struct tMCBHead {
  size_t size;
  unsigned flags;
} tMCBHead;

typedef struct tMCBTail {
  size_t size;
} tMCBTail;

#pragma pack (pop)

typedef struct tAlloc {
  void* pHeapBegin;
  void* pHeapEnd;
  size_t PageSize;
  int (*pMapPages) (struct tAlloc* pAlloc, void* pFirstPage, size_t count);
  void (*pUnmapPages) (struct tAlloc* pAlloc, void* pFirstPage, size_t count);
} tAlloc;

typedef struct tAllocInit {
  void* pHeapBegin;
  size_t HeapSize;
  size_t PageSize;
  int (*pMapPages) (tAlloc* pAlloc, void* pFirstPage, size_t count);
  void (*pUnmapPages) (tAlloc* pAlloc, void* pFirstPage, size_t count);
} tAllocInit;

typedef struct tAllocStatistics {
  unsigned long BlocksAllocated, BlocksFree, BlocksTotal;
  size_t MemoryAllocated, MemoryFree, MemoryTotal;
  size_t LargestFreeSize;
  unsigned long Errors;
  unsigned long TreeHeight;
} tAllocStatistics;

#ifdef __cplusplus
extern "C" {
#endif

extern int    ALLOC_Init (tAlloc* pAlloc, tAllocInit* pAllocInit);
extern void   ALLOC_Done (tAlloc* pAlloc);
extern void*  ALLOC_Alloc (tAlloc* pAlloc, size_t size);
extern void   ALLOC_Free (tAlloc* pAlloc, void* pBuf);
extern void   ALLOC_GetStatistics (tAlloc* pAlloc, tAllocStatistics* pStat);

#ifdef __cplusplus
}
#endif

#endif // _ALLOC_H_
